import "./public-path.js"
import * as monaco from 'monaco-editor';
import init, { Workspace } from "./ruff/crates/ruff_wasm/src/pkg";


/**
 * @param {monaco.editor.IEditor} editor
 * @param {object} config the config, following https://raw.githubusercontent.com/charliermarsh/ruff/main/ruff.schema.json
 * @returns {Disposable}
 */
self.registerRuffDiagnosticProvider = async (editor, config) => {
	await init();
	const workspace = new Workspace(config)
	const model = editor.getModel()
	let codeActionProviderDisposable;

	return model.onDidChangeContent(() => {
		const diagnostics = workspace.check(model.getValue());
		codeActionProviderDisposable?.dispose();
		codeActionProviderDisposable = monaco.languages.registerCodeActionProvider("python", {
			provideCodeActions: function (model, position) {
				const actions = diagnostics
					.filter((check) => position.startLineNumber === check.location.row)
					.filter(({ fix }) => fix)
					.map((check) => ({
						title: check.fix
							? check.fix.message
								? `${check.code}: ${check.fix.message}`
								: `Fix ${check.code}`
							: "Fix",
						id: `fix-${check.code}`,
						kind: "quickfix",
						edit: check.fix
							? {
									edits: check.fix.edits.map((edit) => ({
										resource: model.uri,
										versionId: model.getVersionId(),
										textEdit: {
											range: {
												startLineNumber: edit.location.row,
												startColumn: edit.location.column,
												endLineNumber: edit.end_location.row,
												endColumn: edit.end_location.column,
											},
											text: edit.content || "",
										},
									})),
								}
							: undefined,
					}));
				return { actions, dispose: () => {} };
			},
		});

		monaco.editor.setModelMarkers(
			model,
			"owner",
			diagnostics.map((diagnostic) => ({
				startLineNumber: diagnostic.location.row,
				startColumn: diagnostic.location.column,
				endLineNumber: diagnostic.end_location.row,
				endColumn: diagnostic.end_location.column,
				message: `${diagnostic.code}: ${diagnostic.message}`,
				severity: monaco.MarkerSeverity.Error,
				tags:
					diagnostic.code === "F401" || diagnostic.code === "F841"
						? [monaco.MarkerTag.Unnecessary]
						: [],
			})),
		);
	}
	);
}

self.MonacoEnvironment = {
	getWorkerUrl: function (moduleId, label) {
		if (label === 'json') {
			return window.monacoEditorWebPackResourceBaseUrl + './json.worker.bundle.min.js';
		}
		if (label === 'css' || label === 'scss' || label === 'less') {
			return window.monacoEditorWebPackResourceBaseUrl + './css.worker.bundle.min.js';
		}
		if (label === 'html' || label === 'handlebars' || label === 'razor') {
			return window.monacoEditorWebPackResourceBaseUrl + './html.worker.bundle.min.js';
		}
		if (label === 'typescript' || label === 'javascript') {
			return window.monacoEditorWebPackResourceBaseUrl + './ts.worker.bundle.min.js';
		}
		return window.monacoEditorWebPackResourceBaseUrl + './editor.worker.bundle.min.js';
	}
}

self.monaco = monaco;
